"""
The MIT License (MIT)

Copyright (c) 2015 Zagaran, Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

@author: Zags (Benjamin Zagorsky)
"""

import json
from pymongo import ASCENDING, DESCENDING

from mongolia.constants import ID_KEY, GT
from mongolia.database_object import DatabaseObject
from mongolia.json_codecs import MongoliaJSONEncoder


class DatabaseCollection(list):
    """
    Represent a MongoDB collection as a Python list of DatabaseObjects.
    
    OBJTYPE is a child class of DatabaseObject; this is the type of object
        to be used in the retrieval of database objects; children
        classes of DatabaseCollection should override this attribute
    
    PATH is the database path in the form "database.collection";
        if not None, this overrides the PATH attribute of OBJTYPE;
        if is None, then the PATH attribute of OBJTYPE is used instead;
        children classes of DatabaseCollection can optionally override this
        attribute, but defining PATH through OBJTYPE is preferable
    
    
    Child Class Example:
    
    class User(DatabaseObject):
        PATH = "application.users"
        DEFAULTS = {
            "email": REQUIRED,
            "password": REQUIRED,
            "time_created": datetime.now,
            "name": "anonymous"
        }
        
    class Users(DatabaseCollection):
        OBJTYPE = User
    """
    OBJTYPE = DatabaseObject
    PATH = None
    
    def __init__(self, path=None, objtype=None, query=None, sort_by=ID_KEY, ascending=True,
                 page=0, page_size=None, read_only=False, projection=None, field=None,
                 **kwargs):
        """
        Loads a list of DatabaseObjects from path matching query.  If nothing
        matches the query (possibly because there is nothing in the specified
        mongo collection), the created DatabaseCollection will be an empty
        list and have bool(returned object) == False
        
        NOTE: The path and objtype parameters to this function are to allow
        use of the DatabaseCollection class directly.  However, this class is
        intended for subclassing and children of it should override either the
        OBJTYPE or PATH attribute rather than passing them as parameters here.
        
        WARNING: if you are attempting to grab a particularly large set of
        results (such as an entire collection), your system may run out of
        memory.  In the event of large collections, the number of returned
        results can be reduced either by filtering the results with the query
        parameter or by using pagination via the page_size and page parameters.
        If all you need to do is iterate over the collection, use the `iterator`
        classmethod.
        
        @param path: the path of the database to query, in the form
            "database.colletion"; pass None to use the value of the
            PATH property of the object or, if that is none, the
            PATH property of OBJTYPE
        @param objtype: the object type to use for these DatabaseObjects;
            pass None to use the OBJTYPE property of the class
        @param query: a dictionary specifying key-value pairs that the result
            must match.  If query is None, use kwargs in it's place
        @param sort_by: a key to use for the sort order of the results; ID_KEY
            by default. Can also be a list of pairs [(key, direction), ...], as
            in pymongo's sort function. If set to None, no sort operation is
            applied.
        @param ascending: whether to sort the results in ascending order of
            the sort_by key (if True) or descending order (if False). Ignored if
            sort_by is a list or None.
        @param page: the page number of results to return if pagination is
            being uses; note that if page_size is None, this parameter is
            ignored; page is 0-indexed
        @param page_size: returns only a single page of results, this
            defining the number of results in a page; see also the page
            parameter; if this is None or 0, paging is disabled
        @param read_only: returns the contents as python dictionaries rather than
            DatabaseObjects.  This is not read_only in the sense that the
            returned objects are immutable, but in the sense that they have no
            attached .save() methods, so there is no way to write modifications
            to them back to the database.
        @param projection: specifies fields to return in a projection query. May
            be a list of field names, in which case the ID_KEY field is always
            returned whether or not listed. To prevent ID_KEY from being
            included, use a dict instead (see pymongo documentation for
            projection queries). Specifying this field implies read_only.
        @param field: returns simply the indicated field for each object
            rather than the entire object.  For example, if ID_KEY is passed in
            for this parameter, only the ID's of the collection are returned,
            not the entire contents.  This behaves similarly to read_only
            in that the returned objects cannot be saved to the database if
            they are updated.  Objects that do not have the indicated field
            are omitted from results. Do not combine with the projection parameter.
        @param **kwargs: used as query parameters if query is None
        
        @raise Exception: if path, self.PATH, and self.OBJTYPE.PATH are all
            None; the database path must be defined in at least one of these
        """
        if objtype:
            self.OBJTYPE = objtype
        if path:
            self.PATH = path
        if not query:
            query = kwargs
        if field:
            if field == ID_KEY:
                projection = [ID_KEY]
            else:
                projection = {field: True, ID_KEY: False}
        if projection:
            read_only = True
        results = self.db().find(query, projection=projection)    
        if isinstance(sort_by, list):
            results = results.sort(sort_by)
        elif sort_by:
            results = results.sort(sort_by, ASCENDING if ascending else DESCENDING)
        if page_size:
            results.limit(page_size).skip(page_size * page)
        if field:
            for result in results:
                if field in result:
                    self.append(result[field])
        elif read_only:
            for result in results:
                self.append(result)
        else:
            for result in results:
                self.append(self.OBJTYPE(path=self.PATH, _new_object=result))
    
    @classmethod
    def count(cls, path=None, objtype=None, query=None, **kwargs):
        """
        Like __init__, but simply returns the number of objects that match the
        query rather than returning the objects
        
        NOTE: The path and objtype parameters to this function are to allow
        use of the DatabaseCollection class directly.  However, this class is
        intended for subclassing and children of it should override either the
        OBJTYPE or PATH attribute rather than passing them as parameters here.
        
        @param path: the path of the database to query, in the form
            "database.colletion"; pass None to use the value of the
            PATH property of the object or, if that is none, the
            PATH property of OBJTYPE
        @param objtype: the object type to use for these DatabaseObjects;
            pass None to use the OBJTYPE property of the class
        @param query: a dictionary specifying key-value pairs that the result
            must match.  If query is None, use kwargs in it's place
        @param **kwargs: used as query parameters if query is None
        
        @raise Exception: if path, PATH, and OBJTYPE.PATH are all None;
            the database path must be defined in at least one of these
        """
        if not objtype:
            objtype = cls.OBJTYPE
        if not path:
            path = cls.PATH
        if not query:
            query = kwargs
        return objtype.db(path).find(query).count()
    
    @classmethod
    def get_last(cls, **kwargs):
        """
        Gets the last item from a collection, returning a DatabaseObject.
        This is primarily for collections that use random_id's, as mongo's id's
        are in alphachronological order (alphabetical order of time), and this
        will return the most recent item in that case.
        
        @param **kwargs: forwarded to cls.__init__
        """
        try:
            return cls(page_size=1, ascending=False, **kwargs)[0]
        except IndexError:
            return None
    
    @classmethod
    def iterator(cls, path=None, objtype=None, query=None, page_size=1000, **kwargs):
        """"
        Linear time, constant memory, iterator for a mongo collection.
        
        @param path: the path of the database to query, in the form
            "database.colletion"; pass None to use the value of the
            PATH property of the object or, if that is none, the
            PATH property of OBJTYPE
        @param objtype: the object type to use for these DatabaseObjects;
            pass None to use the OBJTYPE property of the class
        @param query: a dictionary specifying key-value pairs that the result
            must match.  If query is None, use kwargs in it's place
        @param page_size: the number of items to fetch per page of iteration
        @param **kwargs: used as query parameters if query is None
        """
        if not objtype:
            objtype = cls.OBJTYPE
        if not path:
            path = cls.PATH
        db = objtype.db(path)
        if not query:
            query = kwargs
        results = list(db.find(query).sort(ID_KEY, ASCENDING).limit(page_size))
        while results:
            page = [objtype(path=path, _new_object=result) for result in results]
            for obj in page:
                yield obj
            query[ID_KEY] = {GT: results[-1][ID_KEY]}
            results = list(db.find(query).sort(ID_KEY, ASCENDING).limit(page_size))
    
    def db(self):
        """
        Calls the db method of OBJTYPE
        
        NOTE: this function is only properly usable on children classes that
        have overridden either OBJTYPE or PATH
        
        @raise Exception: if self.OBJTYPE.PATH or path are valid
        """
        return self.OBJTYPE.db(self.PATH)
    
    def insert(self, data, **kwargs):
        """
        Calls the create method of OBJTYPE
        
        NOTE: this function is only properly usable on children classes that
        have overridden either OBJTYPE or PATH.
        
        @param data: the data of the new object to be created
        @param **kwargs: forwarded to create
        
        @raise DatabaseConflictError: if there is already an object with that
            ID_KEY and overwrite == False
        @raise MalformedObjectError: if a REQUIRED key of defaults is missing,
            or if the ID_KEY of the object is None and random_id is False
        """
        obj = self.OBJTYPE.create(data, path=self.PATH, **kwargs)
        self.append(obj)
        return obj
    
    def to_json(self):
        """
        Returns the json string of the database collection in utf-8.
        
        Note: ObjectId and datetime.datetime objects are custom-serialized
        using the MongoliaJSONEncoder because they are not natively json-
        serializable.
        """
        return json.dumps(self, cls=MongoliaJSONEncoder, encoding="utf-8")
    
    def _move(self, new_path):
        """
        Moves the collection to a different database path
        
        NOTE: this function is intended for command prompt use only.
        
        WARNING: if execution is interrupted halfway through, the collection will
        be split into multiple pieces.  Furthermore, there is a possible
        duplication of the database object being processed at the time of
        interruption.
        
        @param new_path: the new place for the collection to live, in the format
            "database.collection"
        """
        for elt in self:
            DatabaseObject.create(elt, path=new_path)
        for elt in self:
            elt._collection.remove({ID_KEY: elt[ID_KEY]})



"""
The MIT License (MIT)

Copyright (c) 2015 Zagaran, Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

@author: Zags (Benjamin Zagorsky)
"""

import collections
import json

from logging import log, WARN
from past.builtins import basestring
from mongolia.constants import (ID_KEY, CHILD_TEMPLATE, UPDATE, SET,
    REQUIRED_VALUES, REQUIRED_TYPES, TYPES_TO_CHECK, TEST_DATABASE_NAME)
from mongolia.errors import (TemplateDatabaseError, MalformedObjectError,
    RequiredKeyError, DatabaseConflictError, InvalidKeyError, InvalidTypeError,
    NonexistentObjectError)
from mongolia.json_codecs import MongoliaJSONEncoder, MongoliaJSONDecoder
from mongolia.mongo_connection import CONNECTION, AlertLevel

class DatabaseObject(dict):
    """
    Represent a MongoDB object as a Python dictionary.
        
    PATH is the database path in the form "database.collection"; children
        classes of DatabaseObject should override this attribute.
        PATH is what specifies which collection in mongo an item is stored in.
        PATH SHOULD BE UNIQUE FOR EACH CHILD OF DatabaseObject.
        IF TWO DatabaseObjects ARE CREATED WITH THE SAME PATH, THEIR DATA
        WILL BE STORED IN THE SAME COLLECTION.
    
    DEFAULTS is a dictionary of default values for keys of the dict;
        defaults can be functions; REQUIRED is a special value for a key that
        raises a MalformedObjectError if that key isn't in the dict at save
        time; ; children classes of DatabaseObject can optionally override
        this attribute
    
    
    Child Class Example:
    
    class User(DatabaseObject):
        PATH = "application.users"
        DEFAULTS = {
            "email": REQUIRED,
            "password": REQUIRED,
            "time_created": datetime.now,
            "name": "anonymous"
        }
        
    __getattr__, __setattr__, and __delattr__ have been overridden to behave
    as item accessors.  This means that you can access elements in the
    DatabaseObject by either database_object["key"] or database_object.key;
    database_object["key"] syntax is preferable for use in production code
    since there is no chance of conflicting with any of the methods attached
    to the DatabaseObject.  For example, if your entry is named "copy", you can
    only access it by means of database_object["copy"], as database_object.copy
    gives lookup preference to the .copy() method.  Mostly, the ability to
    use the attribute access is for convenience when interacting with
    DatabaseObjects in an interactive python shell.
    
    """
    PATH = None
    DEFAULTS = {}
    _exists = True
    
    def __init__(self, query=None, path=None, defaults=None, _new_object=None, **kwargs):
        """
        Loads a single database object from path matching query.  If nothing
        matches the query (possibly because there is nothing in the specified
        mongo collection), the created DatabaseObject will be an empty
        dictionary and have bool(returned object) == False.  If more than one
        database object matches the query, a DatabaseConflictError is thrown.
        
        NOTE: The path and defaults parameters to this function are to allow
        use of the DatabaseObject class directly.  However, this class is
        intended for subclassing and children of it should override the PATH
        and DEFAULTS attributes rather than passing them as parameters here.
        
        NOTE: if you pass in a single argument to __init__, this will
        match against ID_KEY.
        
        @param query: a dictionary specifying key-value pairs that the result
            must match.  If query is None, use kwargs in it's place
        @param path: the path of the database to query, in the form
            "database.colletion"; pass None to use the value of the
            PATH property of the object
        @param defaults: the defaults dictionary to use for this object;
            pass None to use the DEFAULTS property of the object
        @param _new_object: internal use only
        @param **kwargs: used as query parameters if query is None
        
        @raise Exception: if path and self.PATH are None; the database path
            must be defined in at least one of these
        @raise TemplateDatabaseError: if PATH is CHILD_TEMPLATE; this
            constant is for children classes that are not meant to be
            used as database accessors themselves, but rather extract
            common functionality used by DatabaseObjects of various collections
        @raise DatabaseConflictError: if PATH is CHILD_TEMPLATE; this
            constant is for children classes that are not meant to be
            used as database accessors themselves, but rather extract
            common functionality used by DatabaseObjects of various collections
        """
        if path:
            dict.__setattr__(self, "PATH", path)
        if defaults:
            dict.__setattr__(self, "DEFAULTS", defaults)
        if self.PATH == CHILD_TEMPLATE:
            raise TemplateDatabaseError()
        if _new_object is not None:
            dict.__init__(self, _new_object)
            return
        if query is None and len(kwargs) > 0:
            query = kwargs
        if query is not None:
            if not isinstance(query, collections.Mapping):
                query = {ID_KEY: query}
            cursor = self.db(path).find(query)
            if cursor.count() > 1:
                raise DatabaseConflictError(('More than one database object ' +
                                             'was found for query "%s"') % (query, ))
            for result in cursor.limit(-1):
                dict.__init__(self, result)
                return
        dict.__setattr__(self, "_exists", False)
    
    @classmethod
    def exists(cls, query=None, path=None, **kwargs):
        """
        Like __init__ but simply returns a boolean as to whether or not the
        object exists, rather than returning the whole object.
        
        NOTE: if you pass in a single argument to exists, this will
        match against ID_KEY.
        
        @param query: a dictionary specifying key-value pairs that the result
            must match.  If query is None, use kwargs in it's place
        @param path: the path of the database to query, in the form
            "database.colletion"; pass None to use the value of the
            PATH property of the object
        @param **kwargs: used as query parameters if query is None
        
        @raise Exception: if path and self.PATH are None; the database path
            must be defined in at least one of these
        """
        if query is None and len(kwargs) > 0:
            query = kwargs
        if query is None:
            return False
        return cls.db(path).find_one(query) is not None
    
    @classmethod
    def create(cls, data, path=None, defaults=None, overwrite=False,
               random_id=False, **kwargs):
        """
        Creates a new database object and stores it in the database
        
        NOTE: The path and defaults parameters to this function are to allow
        use of the DatabaseObject class directly.  However, this class is
        intended for subclassing and children of it should override the PATH
        and DEFAULTS attributes rather than passing them as parameters here.
        
        @param data: dictionary of data that the object should be created with;
            this must follow all mongo rules, as well as have an entry for
            ID_KEY unless random_id == True
        @param path: the path of the database to use, in the form
            "database.collection"
        @param defaults: the defaults dictionary to use for this object
        @param overwrite: if set to true, will overwrite any object in the
            database with the same ID_KEY; if set to false will raise an
            exception if there is another object with the same ID_KEY
        @param random_id: stores the new object with a random value for ID_KEY;
            overwrites data[ID_KEY]
        @param **kwargs: ignored
        
        @raise Exception: if path and self.PATH are None; the database path
            must be defined in at least one of these
        @raise DatabaseConflictError: if there is already an object with that
            ID_KEY and overwrite == False
        @raise MalformedObjectError: if a REQUIRED key of defaults is missing,
            or if the ID_KEY of the object is None and random_id is False
        """
        self = cls(path=path, defaults=defaults, _new_object=data)
        for key, value in self.items():
            if key == ID_KEY:
                continue
            if self.DEFAULTS and key not in self.DEFAULTS:
                self._handle_non_default_key(key, value)
            self._check_type(key, value)
        if random_id and ID_KEY in self:
            dict.__delitem__(self, ID_KEY)
        if not random_id and ID_KEY not in self:
            raise MalformedObjectError("No " + ID_KEY + " key in item")
        if not random_id and not overwrite and self._collection.find_one({ID_KEY: data[ID_KEY]}):
            raise DatabaseConflictError('ID_KEY "%s" already exists in collection %s' %
                                        (data[ID_KEY], self.PATH))
        self._pre_save()
        if ID_KEY in self and overwrite:
            self._collection.replace_one({ID_KEY: self[ID_KEY]}, dict(self), upsert=True)
        else:
            insert_result = self._collection.insert_one(dict(self))
            dict.__setitem__(self, ID_KEY, insert_result.inserted_id)
        return self
    
    @classmethod
    def create_from_json(cls, json_str, ignore_non_defaults=True):
        """
        Creates a database object from a json object.  The intent of this method
        is to allow creating a database object directly from json.
        
        Mongolia will also automatically convert any json values that are
        formatted using the MongoliaJSONEncoder (for ObjectIds and datetime
        objects) back to their native python data types.
        
        Note: if using AngularJS, make sure to pass json back using
        `angular.toJson(obj)` instead of `JSON.stringify(obj)` since angular
        sometimes adds `$$hashkey` to javascript objects and this will cause
        a mongo error due to the "$" prefix in keys.
        
        @param json_str: the json string containing the new object to use for
            creating the new object
        @param ignore_non_defaults: if this is True and the database object
            has non-empty DEFAULTS, then any top-level keys of the create json
            that do not appear in DEFAULTS will also be excluded in creation
        """
        create_dict = json.loads(json_str, cls=MongoliaJSONDecoder, encoding="utf-8")
        # Remove all keys not in DEFAULTS if ignore_non_defaults is True
        if cls.DEFAULTS and ignore_non_defaults:
            for key in frozenset(create_dict).difference(frozenset(cls.DEFAULTS)):
                del create_dict[key]
        
        cls.create(create_dict, random_id=True)
    
    @classmethod
    def db(cls, path=None):
        """
        Returns a pymongo Collection object from the current database connection.
        If the database connection is in test mode, collection will be in the
        test database.
        
        @param path: if is None, the PATH attribute of the current class is used;
            if is not None, this is used instead
        
        @raise Exception: if neither cls.PATH or path are valid
        """
        if cls.PATH is None and path is None:
            raise Exception("No database specified")
        if path is None:
            path = cls.PATH
        if "." not in path:
            raise Exception(('invalid path "%s"; database paths must be ' +
                             'of the form "database.collection"') % (path,))
        if CONNECTION.test_mode:
            return CONNECTION.get_connection()[TEST_DATABASE_NAME][path]
        (db, coll) = path.split('.', 1)
        return CONNECTION.get_connection()[db][coll]
    
    def __getitem__(self, key):
        if not self._exists:
            raise NonexistentObjectError("The object does not exist")
        if key == ID_KEY or key == "ID_KEY":
            return dict.__getitem__(self, ID_KEY)
        elif key in self:
            value = dict.__getitem__(self, key)
            self._check_type(key, value, warning_only=True)
            return value
        try:
            new = self._get_from_defaults(key)
        except RequiredKeyError:
            raise MalformedObjectError("'%s' is a required key of %s" %
                                       (key, type(self).__name__))
        dict.__setitem__(self, key, new)
        return new
    
    def __setitem__(self, key, value):
        if not self._exists:
            raise NonexistentObjectError("The object does not exist")
        if key == ID_KEY or key == "ID_KEY":
            # Do not allow setting ID_KEY directly
            raise KeyError("Do not modify '%s' directly; use rename() instead" % ID_KEY)
        if not isinstance(key, basestring):
            raise InvalidKeyError("documents must have only string keys, key was %s" % key)
        if self.DEFAULTS and key not in self.DEFAULTS:
            self._handle_non_default_key(key, value)
        self._check_type(key, value)
        dict.__setitem__(self, key, value)
    
    def __delitem__(self, key):
        if not self._exists:
            raise NonexistentObjectError("The object does not exist")
        if key == ID_KEY or key == "ID_KEY":
            # Do not allow deleting ID_KEY
            raise KeyError("Do not delete '%s' directly; use rename() instead" % ID_KEY)
        if key in self:
            dict.__delitem__(self, key)
    
    def __getattr__(self, key):
        return self[key]
    
    def __setattr__(self, key, val):
        self[key] = val
        
    def __delattr__(self, key):
        del self[key]
    
    def __dir__(self):
        return sorted(set(dir(type(self)) + self.keys()))
    
    @property
    def _collection(self):
        return self.db(self.PATH)

    def _pre_save(self):
        if not self._exists:
            raise NonexistentObjectError("The object does not exist")
        # Fill in missing defaults by invoking __getitem__ for each key in DEFAULTS
        for key in self.DEFAULTS:
            try:
                self[key]
            except KeyError:
                pass
    
    def save(self):
        """
        Saves the current state of the DatabaseObject to the database.  Fills
        in missing values from defaults before saving.
        
        NOTE: The actual operation here is to overwrite the entry in the
        database with the same ID_KEY.
        
        WARNING: While the save operation itself is atomic, it is not atomic
        with loads and modifications to the object.  You must provide your own
        synchronization if you have multiple threads or processes possibly
        modifying the same database object.  The update method is better from
        a concurrency perspective.
        
        @raise MalformedObjectError: if the object does not provide a value
            for a REQUIRED default
        """
        self._pre_save()
        self._collection.replace_one({ID_KEY: self[ID_KEY]}, dict(self))
    
    def rename(self, new_id):
        """
        Renames the DatabaseObject to have ID_KEY new_id.  This is the only
        way allowed by DatabaseObject to change the ID_KEY of an object.
        Trying to modify ID_KEY in the dictionary will raise an exception.
        
        @param new_id: the new value for ID_KEY
        
        NOTE: This is actually a create and delete.
        
        WARNING: If the system fails during a rename, data may be duplicated.
        """
        old_id = dict.__getitem__(self, ID_KEY)
        dict.__setitem__(self, ID_KEY, new_id)
        self._collection.save(self)
        self._collection.remove({ID_KEY: old_id})
    
    def remove(self):
        """
        Deletes the object from the database
        
        WARNING: This cannot be undone.  Be really careful when deleting
        programatically.  It is recommended to backup your database before
        applying specific deletes.  If your application uses deletes regularly,
        it is strongly recommended that you have a recurring backup system.
        """
        self._collection.remove({ID_KEY: self[ID_KEY]})
        dict.clear(self)
    
    def copy(self, new_id=None, attribute_overrides={}):
        """
        Copies the DatabaseObject under the ID_KEY new_id.
        
        @param new_id: the value for ID_KEY of the copy; if this is none,
            creates the new object with a random ID_KEY
        @param attribute_overrides: dictionary of attribute names -> values that you would like to override with.
        """
        data = dict(self)
        data.update(attribute_overrides)
        if new_id is not None:
            data[ID_KEY] = new_id
            return self.create(data, path=self.PATH)
        else:
            del data[ID_KEY]
            return self.create(data, random_id=True, path=self.PATH)

    def update(self, update_dict=None, raw=False, **kwargs):
        """
        Applies updates both to the database object and to the database via the
        mongo update method with the $set argument.  Use the `raw` keyword to
        perform an arbitrary mongo update query.
        
        WARNING: Raw updates do not perform type checking.
        
        WARNING: While the update operation itself is atomic, it is not atomic
        with loads and modifications to the object.  You must provide your own
        synchronization if you have multiple threads or processes possibly
        modifying the same database object.  While this is safer from a
        concurrency perspective than the access pattern load -> modify -> save
        as it only updates keys specified in the update_dict, it will still
        overwrite updates to those same keys that were made while the object
        was held in memory.
        
        @param update_dict: dictionary of updates to apply
        @param raw: if set to True, uses the contents of update_dict directly
            to perform the update rather than wrapping them in $set.
        @param **kwargs: used as update_dict if no update_dict is None
        """
        if update_dict is None:
            update_dict = kwargs
        if raw:
            self._collection.update_one({ID_KEY: self[ID_KEY]}, update_dict)
            new_data = self._collection.find_one({ID_KEY: self[ID_KEY]})
            dict.clear(self)
            dict.update(self, new_data)
        else:
            for key, value in update_dict.items():
                self._check_type(key, value)
            dict.update(self, update_dict)
            self._collection.update_one({ID_KEY: self[ID_KEY]}, {SET: update_dict})
    
    def to_json(self):
        """
        Returns the json string of the database object in utf-8.
        
        Note: ObjectId and datetime.datetime objects are custom-serialized
        using the MongoliaJSONEncoder because they are not natively json-
        serializable.
        """
        return json.dumps(self, cls=MongoliaJSONEncoder, encoding="utf-8")
    
    def json_update(self, json_str, exclude=[], ignore_non_defaults=True):
        """
        Updates a database object based on a json object.  The intent of this
        method is to allow passing json to an interface which then subsequently
        manipulates the object and then sends back an update.
        
        Mongolia will also automatically convert any json values that were
        initially converted from ObjectId and datetime.datetime objects back
        to their native python object types.
        
        Note: if using AngularJS, make sure to pass json back using
        `angular.toJson(obj)` instead of `JSON.stringify(obj)` since angular
        sometimes adds `$$hashkey` to javascript objects and this will cause
        a mongo error due to the "$" prefix in keys.
        
        @param json_str: the json string containing the new object to use for
            the update
        @param exclude: a list of top-level keys to exclude from the update
            (ID_KEY need not be included in this list; it is automatically
            deleted since it can't be part of a mongo update operation)
        @param ignore_non_defaults: if this is True and the database object
            has non-empty DEFAULTS, then any top-level keys in the update json
            that do not appear in DEFAULTS will also be excluded from the update
        """
        update_dict = json.loads(json_str, cls=MongoliaJSONDecoder, encoding="utf-8")
        # Remove ID_KEY since it can't be part of a mongo update operation
        if ID_KEY in update_dict:
            del update_dict[ID_KEY]
        
        # Remove all keys in the exclude list from the update
        for key in frozenset(exclude).intersection(frozenset(update_dict)):
            del update_dict[key]
        
        # Remove all keys not in DEFAULTS if ignore_non_defaults is True
        if self.DEFAULTS and ignore_non_defaults:
            for key in frozenset(update_dict).difference(frozenset(self.DEFAULTS)):
                del update_dict[key]
        
        self.update(update_dict)
    
    def json_update_fields(self, json_str, fields_to_update):
        """
        Updates the specified fields of a database object based on a json
        object. The intent of this method is to allow passing json to an
        interface which then subsequently manipulates the object and then sends
        back an update for specific fields of the object.
        
        Mongolia will also automatically convert any json values that were
        initially converted from ObjectId and datetime.datetime objects back
        to their native python object types.
        
        Note: if using AngularJS, make sure to pass json back using
        `angular.toJson(obj)` instead of `JSON.stringify(obj)` since angular
        sometimes adds `$$hashkey` to javascript objects and this will cause
        a mongo error due to the "$" prefix in keys.
        
        @param json_str: the json string containing the new object to use for
            the update
        @param fields_to_update: a list of the top-level keys to update; only
            keys included in this list will be update.  Do not include ID_KEY
            in this list since it can't be part of a mongo update operation
        """
        update_dict = json.loads(json_str, cls=MongoliaJSONDecoder, encoding="utf-8")
        update_dict = dict((k, v) for k, v in update_dict.items()
                       if k in fields_to_update and k != ID_KEY)
        self.update(update_dict)
    
    def _get_from_defaults(self, key):
        # If a KeyError is raised here, it is because the key is found in
        # neither the database object nor the DEFAULTS
        if self.DEFAULTS[key] in REQUIRED_VALUES:
            raise RequiredKeyError(key)
        if self.DEFAULTS[key] == UPDATE:
            raise KeyError(key)
        try:
            # Try DEFAULTS as a function
            default = self.DEFAULTS[key]()
        except TypeError:
            # If it fails, treat DEFAULTS entry as a value
            default = self.DEFAULTS[key]
        # If default is a dict or a list, make a copy to avoid passing by reference
        if isinstance(default, list):
            default = list(default)
        if isinstance(default, dict):
            default = dict(default)
        return default
    
    def _handle_non_default_key(self, key, value):
        # There is an attempt to set a key not in DEFAULTS
        if CONNECTION.defaults_handling == AlertLevel.error:
            raise InvalidKeyError("%s not in DEFAULTS for %s" %
                                  (key, type(self).__name__))
        elif CONNECTION.defaults_handling == AlertLevel.warning:
            log(WARN, "%s not in DEFAULTS for %s" % (key, type(self).__name__))
    
    def _check_type(self, key, value, warning_only=False):
        # Check the type of the object against the type in DEFAULTS
        if not self.DEFAULTS or key not in self.DEFAULTS:
            # If the key is not in defaults, there is nothing to compare to
            return
        default = self.DEFAULTS[key]
        if default in list(REQUIRED_TYPES.keys()) and not isinstance(value, REQUIRED_TYPES[default]):
            # Check types of required fields regardless of alert settings
            message = ("value '%s' for key '%s' must be of type %s" %
                       (value, key, REQUIRED_TYPES[default]))
            if warning_only:
                log(WARN, message)
                return
            raise InvalidTypeError(message)
        if default in REQUIRED_VALUES or default == UPDATE:
            # Handle special keys, including a REQUIRED_TYPE default
            # (which was checked above)
            return
        
        if CONNECTION.type_checking == AlertLevel.none:
            # Shortcut return if type checking is disabled
            return
        type_ = DatabaseObject._get_type(default)
        if type_ is None or isinstance(value, type_):
            # The key either matches the type of the default or the default is
            # not one of the types we check; everything is good
            return
        # If we've gotten here, there is a type mismatch: warn or error
        message = ("value '%s' for key '%s' must be of type %s" %
                   (value, key, type_))
        if CONNECTION.type_checking == AlertLevel.error:
            if warning_only:
                log(WARN, message)
                return
            raise InvalidTypeError(message)
        elif CONNECTION.type_checking == AlertLevel.warning:
            log(WARN, message)
    
    @staticmethod
    def _get_type(default):
        for type_ in TYPES_TO_CHECK:
            if isinstance(default, type_):
                return type_
        return None





import datetime
import dateutil
import json
from bson import ObjectId


OBJECTID_IDENTIFIER = "$oid"
ISO_8601_IDENTIFIER = "$iso"

class MongoliaJSONEncoder(json.JSONEncoder):
    """
    Adds the ability to serialize python ObjectId and datetime.datetime objects
    when included in json.dumps(). The results are json objects with their
    respective identifiers as keys and their serialized values as values.
    
    ObjectId objects are serialized as strings, and datetime.datetime objects
    are serialized to the standard ISO 8601 format.
    
    >>> json.dumps(ObjectId('5717fc0d78ba2f1d6c41919a'), cls=MongoliaJSONEncoder)
    '{OBJECTID_IDENTIFIER: "5717fc0d78ba2f1d6c41919a"}'
    
    >>> json.dumps(datetime.datetime(2016, 4, 20, 18, 28, 12), cls=MongoliaJSONEncoder)
    '{ISO_8601_IDENTIFIER: "2016-04-20T18:28:12"}'
    """
    def default(self, o):
        if isinstance(o, ObjectId):
            return {
                OBJECTID_IDENTIFIER: str(o)
            }
        if isinstance(o, datetime.date) or isinstance(o, datetime.datetime):
            return {
                ISO_8601_IDENTIFIER: o.isoformat()
            }
        return super(MongoliaJSONEncoder, self).default(o)


class MongoliaJSONDecoder(json.JSONDecoder):
    """
    Adds the ability to deserialize Mongolia's json representations of python
    ObjectId and datetime.datetime objects when included in json.loads().
    
    >>> json.loads('{OBJECTID_IDENTIFIER: "5717fc0d78ba2f1d6c41919a"}', cls=MongoliaJSONDecoder)
    ObjectId('5717fc0d78ba2f1d6c41919a')
    
    >>> json.loads('{ISO_8601_IDENTIFIER: "2016-04-20T18:28:12"}', cls=MongoliaJSONDecoder)
    datetime.datetime(2016, 4, 20, 18, 28, 12)
    """
    def __init__(self, *args, **kwargs):
        super(MongoliaJSONDecoder, self).__init__(object_hook=self.convert, *args, **kwargs)
    
    def convert(self, o):
        if OBJECTID_IDENTIFIER in o:
            return ObjectId(o[OBJECTID_IDENTIFIER])
        if ISO_8601_IDENTIFIER in o:
            return dateutil.parser.parse(o[ISO_8601_IDENTIFIER])
        return o


"""
The MIT License (MIT)

Copyright (c) 2015 Zagaran, Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

@author: Zags (Benjamin Zagorsky)
"""
from bson import ObjectId
from datetime import datetime
from past.builtins import basestring

""" Special key required by mongo for all DatabaseObjects; uniquely
    identifies DatabaseObjects """
ID_KEY = "_id"

""" Not used in current library version """
INC = "$inc"

""" Greater than argument to mongo query """
GT = "$gt"

""" Set argument for mongo update """
SET = "$set"

""" List of types to check for under Connection.type_checking """
TYPES_TO_CHECK = [basestring, int, float, list, dict]

""" Indicates that a key in DatabaseObject.DEFAULTS is required """
REQUIRED = "__required__"

""" Special values for keys in DatabaseObject.DEFAULTS that are treated the same
    as REQUIRED, but are also contain type information for type checking.
    A key that has a REQUIRED_TYPE will raise an error if the wrong type of
    data is set for that variable, even if type checking is disabled. """
REQUIRED_STRING = "__required__string"
REQUIRED_INT = "__required__int"
REQUIRED_FLOAT = "__required__float"
REQUIRED_LIST = "__required__list"
REQUIRED_DICT = "__required__dict"
REQUIRED_DATETIME = "__required__datetime"
REQUIRED_OBJECTID = "__required__object_id"

REQUIRED_VALUES = [REQUIRED, REQUIRED_STRING, REQUIRED_INT, REQUIRED_FLOAT,
                   REQUIRED_LIST, REQUIRED_DICT, REQUIRED_DATETIME, REQUIRED_OBJECTID]

REQUIRED_TYPES = {
    REQUIRED_STRING: basestring,
    REQUIRED_INT: int,
    REQUIRED_FLOAT: float,
    REQUIRED_LIST: list,
    REQUIRED_DICT: dict,
    REQUIRED_DATETIME: datetime,
    REQUIRED_OBJECTID: ObjectId
}

""" Indicates that a key in DatabaseObject.DEFAULTS is to be used in update
    operations; treated as REQUIRED in current library version """
UPDATE = "__update__"

""" PATH definition to be used by DatabaseObject children classes that are
    not meant to be used as database accessors themselves, but rather extract
    common functionality used by DatabaseObjects of various collections"""
CHILD_TEMPLATE = "CHILD_TEMPLATE"

""" Name of the database used in test mode """
TEST_DATABASE_NAME = "__MONGOLIA_TEST_DATABASE__"



"""
The MIT License (MIT)

Copyright (c) 2015 Zagaran, Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

@author: Zags (Benjamin Zagorsky)
"""

from pymongo import MongoClient
from pymongo.errors import AutoReconnect, ConnectionFailure, ServerSelectionTimeoutError
from mongolia.errors import DatabaseIsDownError
from mongolia.constants import TEST_DATABASE_NAME


class AlertLevel(object):
    """
    Options for the arguments of set_defaults_handling or set_type_checking:
        error: raise an exception
        warning: logs a warning
        none: take no extra action
    """
    error = "error"
    warning = "warning"
    none = None


class MongoConnection(object):
    """
    MongoConnecton is an object that lazily loads a pymongo.MongoClient;
    the MongoClient is not actually initialized until the user gets it the first
    time through get_connection, explicitly connects to it through connect,
    or authenticates it through authenticate.
    """
    __connection = None
    defaults_handling = AlertLevel.none
    type_checking = AlertLevel.none
    test_mode = False
    
    def get_connection(self):
        """ Returns the the current MongoClient,
            or creates a new one if there isn't one yet"""
        if self.__connection is None:
            self.connect()
        return self.__connection
    
    def connect(self, host=None, port=None, connect=False, **kwargs):
        """ Explicitly creates the MongoClient; this method must be used
            in order to specify a non-default host or port to the MongoClient.
            Takes arguments identical to MongoClient.__init__"""
        try:
            self.__connection = MongoClient(host=host, port=port, connect=connect, **kwargs)
        except (AutoReconnect, ConnectionFailure, ServerSelectionTimeoutError):
            raise DatabaseIsDownError("No mongod process is running.")
    
    def authenticate(self, username, password, db=None):
        """ Authenticates the MongoClient with the passed username and password """
        if db is None:
            return self.get_connection().admin.authenticate(username, password)
        return self.get_connection()[db].authenticate(username, password)
    
    def add_user(self, name, password=None, read_only=None, db=None, **kwargs):
        """ Adds a user that can be used for authentication """
        if db is None:
            return self.get_connection().admin.add_user(
                    name, password=password, read_only=read_only, **kwargs)
        return self.get_connection()[db].add_user(
                    name, password=password, read_only=read_only, **kwargs)
    
    def set_test_mode(self, test_mode=True):
        """ Sets mongolia to use a test database instead of the actual database """
        self.test_mode = test_mode

# TODO: allow multiple simultaneous connections
CONNECTION = MongoConnection()


def connect_to_database(host=None, port=None, connect=False, **kwargs):
    """
    Explicitly begins a database connection for the application
    (if this function is not called, a connection is created when
    it is first needed).  Takes arguments identical to
    pymongo.MongoClient.__init__
    
    @param host: the hostname to connect to
    @param port: the port to connect to
    @param connect:  if True, immediately begin connecting to MongoDB in the
        background; otherwise connect on the first operation
    """
    return CONNECTION.connect(host=host, port=port, connect=connect, **kwargs)

def authenticate_connection(username, password, db=None):
    """
    Authenticates the current database connection with the passed username
    and password.  If the database connection uses all default parameters,
    this can be called without connect_to_database.  Otherwise, it should
    be preceded by a connect_to_database call.
    
    @param username: the username with which you authenticate; must match
        a user registered in the database
    @param password: the password of that user
    @param db: the database the user is authenticated to access.  Passing None
    (the default) means authenticating against the admin database, which
    gives the connection access to all databases
    
    Example; connecting to all databases locally:
        connect_to_database()
        authenticate_connection("username", "password")
    
    Example; connecting to a particular database of a remote server:
        connect_to_database(host="example.com", port="12345")
        authenticate_connection("username", "password", db="somedb")
    
    """
    return CONNECTION.authenticate(username, password, db=db)

def set_defaults_handling(alert_level):
    """
    When DEFAULTS have been specified for a DatabaseObject and a key not in
    the specified DEFAULTS is modified, there are three options, detailed below.
    This function sets the behavior globally for the active system.
    
    @param alert_level: this parameter takes one of three options:
        AlertLevel.error: raises an InvalidKeyError if a non-default key is set.
            Use this option if you are using DEFAULTS as strict database
            schemas, and only things in DEFAULTS should appear in the database.
        AlertLevel.warning (default): prints a warning if a non-default key is
            set.  Use this option if you are using DEFAULTS as strict database
            schemas, but data should still be stored in the event of a mismatch.
        AlertLevel.none: does nothing if a non-default key is set.  Use this
            option if you are just using DEFAULTS to populate default values
            and not as strict database schemas.
    """
    CONNECTION.defaults_handling = alert_level

def set_type_checking(alert_level):
    """
    When DEFUALTS have been specified for a DatabaseObject and a key that is
    in DEFAULTS is saved, there are three options for type checking of the
    stored value against it's default value.  Note: this only works for default
    values that are of one of the following types: int, float, str, list, dict.
    If the value for a key in DEFAULTS is REQUIRED, none, or some other type,
    type checking will be disabled for that key.
    
    The exception to this is if a key is one of the special REQUIRED_TYPE
    variables (such as REQUIRED_INT), in which case the value will be checked
    against the type specified by the REQURIED_TYPE.
    
    NOTE: A key that has a REQUIRED_TYPE will raise an error if the wrong type
    of data is set for that variable, even if type checking is set to "warning"
    or "none".
    
    @param alert_level: this parameter takes one of three options:
        AlertLevel.error: raises an InvalidTypeError if the default value of a
            key is of a different type than the value being set.
        AlertLevel.warning: logs a warning if the default value for a key is of
            a different type than the value being set.
        AlertLevel.none: does nothing if the default value of a key is of a
            different type than the value being set for that key.
    """
    CONNECTION.type_checking = alert_level

def add_user(name, password=None, read_only=None, db=None, **kwargs):
    """
    Adds a user that can be used for authentication.
    
    @param name: the name of the user to create
    @param passowrd: the password of the user to create. Can not be used with
        the userSource argument.
    @param read_only: if True the user will be read only
    @param db: the database the user is authenticated to access.  Passing None
        (the default) means add the user to the admin database, which gives the
        user access to all databases
    @param **kwargs: forwarded to pymongo.database.add_user
    
    Example; adding a user with full database access:
        add_user("username", "password")
    
    Example; adding a user with read only privilage on a partiucalr database:
        add_user("username", "password", read_only=True, db="somedb")
    
    NOTE: This function will only work if mongo is being run unauthenticated
    or you have already authenticated with another user with appropriate
    privileges to add a user to the specified database.
    """
    return CONNECTION.add_user(name, password=password, read_only=read_only, db=db, **kwargs)

def add_superuser(name, password, **kwargs):
    """
    Adds a user with userAdminAnyDatabase role to mongo.
    
    @param name: the name of the user to create
    @param passowrd: the password of the user to create. Can not be used with
        the userSource argument.
    @param **kwargs: forwarded to pymongo.database.add_user
    """
    return CONNECTION.add_user(
            name, password=password,
            roles=["userAdminAnyDatabase", "readWriteAnyDatabase", "root", "backup", "restore"], **kwargs
    )

def list_database(db=None):
    """
    Lists the names of either the databases on the machine or the collections
    of a particular database
    
    @param db: the database for which to list the collection names;
        if db is None, then it lists all databases instead
        the contents of the database with the name passed in db
    """
    if db is None:
        return CONNECTION.get_connection().database_names()
    return CONNECTION.get_connection()[db].collection_names()


def set_test_mode(test_mode=True):
    """
    Sets test mode in which mongolia saves all objects to a temporary database
    called __MONGOLIA_TEST_DATABASE__.
    
    @param test_mode: set True to enable test mode and False to disable it
    
    WARNING: Make sure nothing is in the mongo database __MONGOLIA_TEST_DATABASE__
    as this database gets dropped frequently.
    
    WARNING: Do not enable test mode in production code.  This will cause your
    data to be stored in the test database.
    """
    CONNECTION.set_test_mode(test_mode)

def drop_test_database():
    """
    Drops the database named __MONGOLIA_TEST_DATABASE__ (which is used while
    test mode is active).  This should be run at least at the beginning and end
    of a test suite, if not also between each test.
    """
    CONNECTION.get_connection().drop_database(TEST_DATABASE_NAME)



"""
The MIT License (MIT)

Copyright (c) 2015 Zagaran, Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

@author: Zags (Benjamin Zagorsky)
"""

import unittest2
import sys
from mongolia.mongo_connection import set_test_mode, drop_test_database

class MongoliaTestCase(unittest2.TestCase):
    """ Child class of untitest.TestCase that does the following:
            * Sets the Mongolia connection to test mode before starting
            * Sets the Mongolia connection out of test mode after finishing
            * Drops the test database after each test
        The result of this is that each test case will run on a mongo database
        containing only what the setUp method and the test case itself have
        created.
    """
    
    @classmethod
    def setUpClass(cls):
        set_test_mode(True)
        super(MongoliaTestCase, cls).setUpClass()
    
    @classmethod
    def tearDownClass(cls):
        set_test_mode(False)
        super(MongoliaTestCase, cls).tearDownClass()
    
    def __call__(self, result=None):
        """ Adds extra setup and teardown by overriding the __call__ method
            so that children of this class can define setUp and tearDown
            without needing to call the corresonding super methods """
        try:
            self._mongolia_test_setup()
        except Exception:
            result.addError(self, sys.exc_info())
            return
        super(MongoliaTestCase, self).__call__(result)
        try:
            self._mongolia_test_teardown()
        except Exception:
            result.addError(self, sys.exc_info())
            return
    
    def _mongolia_test_setup(self):
        set_test_mode(True)
    
    def _mongolia_test_teardown(self):
        drop_test_database()
